=begin

Hello.

I'm sharing a little script with everybody that does export & import camera settings, making transfers between files possible.
As I was not able to find any script that can do this (though "Camera Statistics" provides the raw numbers, it requires quite a bit of copy&paste efforts), I went ahead and wrote it myself.

So it's basically tackling an issue we have quite frequently in the office, when we have several versions of SU model files, dating back to different dates and occasions. And then sometime the client asks for a before/after comparison, for what we have to retain older files and retrofit them with newer camera positions (or do the same thing vice-verse). 

The scripting might be ruby-beginner-style, but is seems to be working on our SU 8 Win7 systems.

It provides basic export options (choose between exporting "all scenes", "current scene" "selected scene") and import options (update camera of scenes with identical scene name)

If somebody find this helpful, feel free to download and use it and let me know if you like it.
Also feel free to enhance the script further.

Best regards.
Martin

----------------------------------------------------------------------------

# Name :          camera_export_import.rb 1.0
# Description :   exports and imports camera settings (scenes).
# Author :        Martin Pfirrmann
# Usage :         1. export cameras via "Plugins" / "Camera Export/Import"
#                 2. import cameras via "Plugins" / "Camera Export/Import"
# Date :          2012.08.27
# Type :          Tool
# History:        1.0 (2012.08.27) - first version
				  1.1 (2012.08.29) - second version - correction suggested via http://forums.sketchucation.com/viewtopic.php?f=180&t=18556&start=0 

----------------------------------------------------------------------------

=end


require 'sketchup.rb'
 
module NN_Camera_Export_Import
  
  unless file_loaded?( __FILE__ )
    plugins_menu = UI.menu("Plugins")
	submenu = plugins_menu.add_submenu("||| Camera Export/Import")
	submenu.add_item("Export Camera Settings") { self.run_export_camera }
	submenu.add_item("Import Camera Settings") { self.run_import_camera }
  end
  
  # Use instance variables inside modules instead of global variables.
  @my_variable = 'Hi there! :)'
 
   #Export script ---------------------------------------------------------------
def self.run_export_camera

	model = Sketchup.active_model
	pages   = model.pages
	
	UI.messagebox("Please select where you want to save your file.")
	mypath = UI.savepanel("Save the camera settings", "", "SU_camera_export.txt")
	return if mypath.nil?
	mypath = File.expand_path(mypath)
	myfile = File.new(mypath, "w")
	
	#myfile = File.new("D:\SU_camera_export.txt", "w")
	
	# DIALOG 1: all, current, select 
	question_1 = ["Which Cameras do you want to export?"]
	defaults_1 = ["all"]
	list_1 = ["all|current page|selected"]
	input_1 = UI.inputbox question_1, defaults_1, list_1, "Camera Exporter"
	
	export_option = input_1[0]
	
	if (export_option) == "all"
		#UI.messagebox "all cams"
	elsif (export_option) == "current page"
		#UI.messagebox "current cams"
		
		page = pages.selected_page
		selected_page = page.name
		array_pagenames = selected_page.to_s
			
	# DIALOG 2: selected pages 
	elsif (export_option) == "selected"
		
		array_page_names = []
		array_defaults = []
		array_list = []
		
		default = "yes"
		list = "yes|no"
		
		pages.each do |page|
			model.pages.selected_page = page
			name = page.name.to_s
			
			array_page_names.push(name.to_s)
			array_defaults.push(default.to_s)
			array_list.push(list.to_s)
		end
			
		input_selected_cameras = UI.inputbox array_page_names, array_defaults, array_list, "Select Cameras/Scenes to export."
	
		#puts array_page_names.to_s
		#puts input_selected_cameras.to_s
		
		# pages included
		array_pagenames = []
		count_1 = 0
		array_page_names.each do |n|
			if input_selected_cameras[count_1] == "yes"
				array_pagenames.push(n)
			end
			count_1 = count_1 + 1
			#puts count_1.to_s
			#puts array_pagenames.to_s
		end
	end

	puts "Scenes to be exported: " + array_pagenames.to_s


	# - start export for pages
	c = 1
	pages.each do |page|
		model.pages.selected_page = page
        cam  = page.camera
        name = page.name
		
		# - export options / which pages should be exported -----------------------
		if (export_option) == "all" || array_pagenames.include?(name.to_s)
 		
			perspective_yesno = 0
			my_fov = 0
			if(cam.perspective?)
				perspective_yesno = 1
				my_fov = cam.fov
			else
				perspective_yesno = 0
				my_fov = 0
			end

			# push into array
			my_sep = "xxxxx"
			new_array_entry = [page.name,my_sep,cam.eye.x.to_f,my_sep,cam.eye.y.to_f,my_sep,cam.eye.z.to_f,my_sep,cam.target.x.to_f,my_sep,cam.target.y.to_f,my_sep,cam.target.z.to_f,my_sep,cam.up.x,my_sep,cam.up.y,my_sep,cam.up.z,my_sep,perspective_yesno,my_sep,my_fov]
			
			#puts cam.eye.x
			#puts cam.eye.x.to_f
			#puts cam.eye.y
			#puts cam.eye.z

			#write to file
			myfile.write(new_array_entry.to_s) 
			myfile.write("\n") 
			c = c + 1
		end
	end 
	myfile.close
	
	# success message
	exported_cams = c -1
	if exported_cams > 1
		UI.messagebox exported_cams.to_s + " Cameras were successfully exported to #{mypath}"
	else
		UI.messagebox exported_cams.to_s + " Camera was successfully exported to #{mypath}"
	end
 end
 

 
 # Import script ---------------------------------------------------------------
 
def self.run_import_camera
 	
	# update cameras of pages with the same name?
	how_to_import = UI.messagebox "Would you like to update the Camera if Scene Names are identical?", MB_YESNO
	puts how_to_import
	
	value = UI.openpanel "Select text file" 
	import_file = File.new( value.to_s, "r" )   
	camera_info_file = IO.readlines(value.to_s)
	i = 1 #import counter
	u = 1 #update counter
    
	camera_info_file.each do |text_line|
		#UI.messagebox "Line " + i.to_s + " " + text_line.to_s
 		
		splitted_array = text_line.split('xxxxx')
		elements = splitted_array.length
		#puts "element count: " + elements.to_s
		
		# page name
		my_page_name = splitted_array[0]
		#puts "my_page_name: "+ my_page_name
	  
	    # eye xyz
		my_eye_x  = splitted_array[1]
		#puts "my_eye_x: "+ my_eye_x.to_s
		
		my_eye_y  = splitted_array[2]
		#puts "my_eye_y: "+ my_eye_y.to_s
		
		my_eye_z  = splitted_array[3]
		#puts "my_eye_z: "+ my_eye_z.to_s
		
		# target xyz
		
		my_target_x  = splitted_array[4]
		#puts "my_target_x: "+ my_target_x.to_s
		
		my_target_y  = splitted_array[5]
		#puts "my_target_y: "+ my_target_y.to_s
		
		my_target_z  = splitted_array[6]
		#puts "my_target_z: "+ my_target_z.to_s
		
	  # up xyz
		
		my_up_x  = splitted_array[7]
		#puts "my_up_x: "+ my_up_x.to_s
		
		my_up_y  = splitted_array[8]
		#puts "my_up_y: "+ my_up_y.to_s
		
		my_up_z  = splitted_array[9]
		#puts "my_up_z: "+ my_up_z.to_s
	  
		# perspective + fov
	  
		my_perspective = splitted_array[10]
		#puts "my_perspective: " + my_perspective.to_s
	  
		my_fov =  splitted_array[11]
		#puts "my_fov: " + my_fov.to_s

		camera_eye = [my_eye_x.to_f, my_eye_y.to_f, my_eye_z.to_f]
		camera_target = [my_target_x.to_f, my_target_y.to_f,my_target_z.to_f]
		camera_up = [my_up_x.to_f, my_up_y.to_f, my_up_z.to_f]
		
		# create new camera
		my_camera = Sketchup::Camera.new camera_eye, camera_target, camera_up
		my_camera.fov = my_fov.to_f
	
		view = Sketchup.active_model.active_view
		view.camera = my_camera
		
		model = Sketchup.active_model
		pages = model.pages
		
		# all page names in array ------------
		array_page_names_newfile = []
		pages.each { |s| array_page_names_newfile.push(s.name) }
		
		# page name exisiting? ------------------------------------
		updatecase = 0
		if array_page_names_newfile.include?(my_page_name.to_s)
			updatecase = 1
		end
		
		# how to handle - update or new? ------------------------------------
		
		if updatecase == 1 # exiting page name
		
			if how_to_import == 6 # update camera of page
				#UI.messagebox "update existing page"
				page = pages[my_page_name.to_s]
				status = page.update(1) 
				u = u + 1
			else # create new page
				#UI.messagebox "create new page"
				page = pages.add my_page_name.to_s
				description = page.description = "imported camera settings"
				status = page.use_camera=true
				i = i + 1
			end
					
		else # create new page
			#UI.messagebox "create new page"
			page = pages.add my_page_name.to_s
			description = page.description = "imported camera settings"
			status = page.use_camera=true
			i = i + 1
		end	
		
    end
 
	import_file.close
 
	# success message -----------------------------------------
	imported_cams = i -1
	updated_cams = u -1
		
	message_updated = String.new("")	
	message_imported = String.new("")	
	
	if 	updated_cams > 1
		message_updated = updated_cams.to_s + " Cameras were updated."
	elsif updated_cams == 1
		message_updated = updated_cams.to_s + " Camera was updated."
	elsif updated_cams == 0
		message_updated = 0
	end
	
	if imported_cams > 1
		message_imported = imported_cams.to_s + " new Cameras were imported."
	elsif imported_cams == 1
		message_imported = imported_cams.to_s + " new Camera was imported."
	elsif imported_cams == 0
		message_imported = 0
	end
	
	puts message_imported
	puts message_updated
	
	if message_updated != 0
		success_message = message_imported.to_s
		if message_updated != 0
			success_message = success_message.to_s + "\n" + message_updated.to_s
		end
	elsif message_updated == 0
		if message_updated != 0
			success_message = success_message.to_s + message_updated.to_s
		end
	end
	
	UI.messagebox success_message.to_s
 
end
 
  file_loaded( __FILE__ )
 
end # module

  
